program dxdemo;

uses Windows, SysUtils, Messages, Direct3D8, D3DX8, flashengine;

type

  PColorRec = ^TColorRec;
  TColorRec = packed record
    case longword of
      0: (Color: cardinal);
      2: (HiWord, LoWord: Word);
      3: (B, G, R, A: Byte);
    end;

  PColorArray = ^TColorArray;
  TColorArray = array [0..0] of cardinal;

  TdxColorTexVertexNormal = packed record
    x, y, z: single;
    nx, ny, nz: single;
    color: longword;
    tu, tv: single;
  end;
  PdxColorTexVertexNormal = ^TdxColorTexVertexNormal;

  TdxColorTexVertexNormalArray = array[0..1000] of TdxColorTexVertexNormal;
  PdxColorTexVertexNormalArray = ^TdxColorTexVertexNormalArray;

var
  g_hwnd: HWND = 0;
  dev: IDirect3DDevice8 = nil;
  ////////////////////////////////////
  hr: HResult;
  hdc: THandle;
  d3dpp: TD3DPresentParameters;

  finished : Boolean;
  FPSCount : Integer = 0;            // Counter for FPS
  CurrentTime, DeltaTime, FrameTime: double;

  Mesh: ID3DXMesh;
  Tex: IDirect3DTexture8;
  TexSize: integer = 256;

  Player: cardinal;

function CheckDxError(Res: HResult): boolean;
begin
  Result := FAILED(Res);
end;

function ColorToD3DColor(const AColor: cardinal): TD3DColorValue;
begin
  Result.r := TColorRec(AColor).R / $FF;
  Result.g := TColorRec(AColor).G / $FF;
  Result.b := TColorRec(AColor).B / $FF;
  Result.a := TColorRec(AColor).A / $FF;
end;

function GetPresentParameters(hdc: THandle): TD3DPresentParameters;
begin
  FillChar(Result, SizeOf(Result), 0);
  Result.Windowed := true;
  Result.hDeviceWindow:= g_hwnd;
  Result.BackBufferWidth := 800;
  Result.BackBufferHeight := 600;
  if Windows.GetDeviceCaps(hdc, BITSPIXEL) = 16 then
    Result.BackBufferFormat:= D3DFMT_R5G6B5
  else
    Result.BackBufferFormat:= D3DFMT_X8R8G8B8;
  Result.BackBufferCount := 1;
  Result.EnableAutoDepthStencil := True;
  Result.AutoDepthStencilFormat := D3DFMT_D16;
  Result.SwapEffect := D3DSWAPEFFECT_DISCARD;
end;

procedure InitFlash;
begin
  Player := LoadMovie(PChar(ExtractFilePath(ParamStr(0)) + 'demo.flv'), TexSize, TexSize);
end;

procedure FreeFlash;
begin
  FreeMovie(Player);
end;

procedure InitScene;
const
  tileFactor = 1;
  width = 10;
  length = 10;
var
  cloneMesh: ID3DXMesh;
  tempBuffer: IDirect3DVertexBuffer8;
  i, numVerts: integer;
  verts: PdxColorTexVertexNormal;
begin
  { init scene }
//  D3DXLoadMeshFromX(PChar(ExtractFilePath(ParamStr(0)) + 'demo.x'), D3DXMESH_SYSTEMMEM, dev, nil, nil, nil, Mesh);
  D3DXCreateBox(dev, width, width, length, Mesh, nil);
  Mesh.CloneMeshFVF(0, D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1 or D3DFVF_DIFFUSE, dev, cloneMesh);
  D3DXComputeNormals(cloneMesh);

  cloneMesh.GetVertexBuffer(tempBuffer);
  if Assigned(tempBuffer) then
  begin
    numVerts := cloneMesh.GetNumVertices();
    tempBuffer.Lock(0, 0, PByte(verts), 0);
    // Generate texture coordinates for the box
    for i := 0 to numVerts - 1 do
    begin
      verts.tu := (tileFactor) * (verts.x - width * 0.5) / width;
      verts.tv := 1.0 - (tileFactor) * (verts.z - length * 0.5) / length;
      verts.color := $ffffffff;
      Inc(verts);
    end;
    tempBuffer.Unlock();
    tempBuffer := nil;
  end;
  Mesh := nil;
  Mesh := cloneMesh;
  D3DXCreateTexture(dev, TexSize, TexSize, 0, 0, D3DFMT_A8R8G8B8, D3DPOOL_MANAGED, Tex);
end;

procedure DrawScene;
var
  matView, matWorld, matTrans: TD3DXMATRIX;
  pLight: TD3DLight8;
  Material: TD3DMaterial8;
  Surface: TD3DLockedRect;
  Desc: TD3DSurfaceDesc;
  FrameBits: PColorArray;
  i: integer;
begin
  if (dev <> nil) then
  begin
    { update texture from flash }
    if (Player <> 0) and (Assigned(Tex)) and (FPSCount mod 5 = 0)  then
    begin
      if not CheckDxError(Tex.LockRect(0, Surface, nil, 0)) then
      begin
        GrabCurrentFrame(Player, Pointer(FrameBits));
        if FrameBits <> nil then
        begin
          Move(FrameBits^, Surface.pBits^, TexSize * TexSize * 4);
        end;
        Tex.UnlockRect(0);
      end;
    end;

    { render }
    dev.BeginScene;

    D3DXMatrixPerspectiveFovLH(matView, 3.14 / 4, 640 / 480, 1.0, 1000);
    dev.SetTransform(D3DTS_PROJECTION, matView);
    D3DXMatrixLookAtLH(matView, D3DXVector3(0, 30, 0), D3DXVector3(0, 0, 0), D3DXVector3(0, 0, 1));
    dev.SetTransform(D3DTS_VIEW, matView);

    ZeroMemory(@pLight, SizeOf(pLight));
    pLight._Type := D3DLIGHT_POINT;
    pLight.Diffuse := D3DXColor(1, 1, 1, 1);
    pLight.Position := D3DXVector3(10, 40, 5);
    pLight.Direction := D3DXVector3(0, 1, 0);
    pLight.Range := 100.0;
    pLight.Attenuation0 := 0.5;
    dev.SetLight(0, pLight);

    D3DXMatrixRotationX(matWorld, CurrentTime * 0.5);
    D3DXMatrixRotationY(matTrans, CurrentTime * 0.5);
//    D3DXMatrixTranslation(matTrans, 0, -10, 0);
    D3DXMatrixMultiply(matWorld, matWorld, matTrans);
    dev.SetTransform(D3DTS_WORLD, matWorld);

    dev.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(40, 40, 40), 1, 0);

    dev.SetRenderState(D3DRS_LIGHTING, iFALSE);
    dev.SetRenderState(D3DRS_AMBIENT, D3DCOLOR_COLORVALUE(0.5, 0.5, 0.5, 1.0));

    dev.LightEnable(0, FALSE);

    dev.SetRenderState(D3DRS_ALPHABLENDENABLE, iTRUE);
    dev.SetRenderState(D3DRS_ALPHATESTENABLE,  iFALSE);
    dev.SetRenderState(D3DRS_ALPHAREF, 1);

    dev.SetRenderState(D3DRS_SHADEMODE, D3DSHADE_GOURAUD);

    dev.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

    dev.SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
    dev.SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    dev.SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
    dev.SetTextureStageState(0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE);
    dev.SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
    dev.SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

    dev.SetTextureStageState(0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR);
    dev.SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
    dev.SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);

    dev.SetRenderState(D3DRS_NORMALIZENORMALS, iTRUE);


    Material.Ambient := ColorToD3dColor($FFFFFFFF);
    Material.Diffuse := ColorToD3dColor($FFFFFFFF);
    Material.Specular := ColorToD3DColor($FFFFFFFF);
    Material.Emissive := ColorToD3DColor($FF505050);
    Material.Power := 0;
    dev.SetMaterial(Material);

    dev.SetTexture(0, Tex);

    Mesh.DrawSubset(0);

    dev.EndScene;

    hr:= dev.Present(nil, nil, 0, nil);
    while (hr = D3DERR_DEVICELOST) do
    begin
      repeat
        Sleep(1000);
        hr:= dev.TestCooperativeLevel;
      until (hr = D3DERR_DEVICENOTRESET);

      hdc:= GetDC(g_hwnd);

      d3dpp:= GetPresentParameters(hdc);
      hr := dev.Reset(d3dpp);
      ReleaseDC(g_hwnd, hdc);

      if (FAILED(hr)) then hr:= D3DERR_DEVICELOST else Break;
    end;
    ValidateRect(g_hwnd, nil);
    Exit;
  end;
end;

function WndProc(hwnd: HWND; uMsg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  case uMsg of
    WM_CREATE:
      begin
        // Insert stuff you want executed when the program starts
      end;
    WM_CLOSE:
      begin
        PostQuitMessage(0);
        Result := 0
      end;
    WM_KEYDOWN:       // Set the pressed key (wparam) to equal true so we can check if its pressed
      begin
        if wParam = VK_ESCAPE then
          Finished := true;
        Result := 0;
      end;
    WM_ERASEBKGND:
      begin
        Result := 1;
      end;
    WM_PAINT:
      begin
        Result := DefWindowProc(hwnd, uMsg, wParam, lParam);
        DrawScene;
      end;
    WM_SETCURSOR:
      begin
        SetCursor(0);
        Result := 1;
      end;
  end;
  Result:= DefWindowProc(hwnd, uMsg, wParam, lParam);
end;

const
  APP_NAME = 'TRIANGLE';

var
  wc: TWndClass;
  pEnum: IDirect3D8 = nil;
  msg: TMsg;
  LastTime: double;
  QFreq, QTimerBefore, QTimerAfter: Int64;
begin
  // Register the window class for the main window.
  wc.style:= 0;
  wc.lpfnWndProc:= @WndProc;
  wc.cbClsExtra:= 0;
  wc.cbWndExtra:= 0;
  wc.hInstance:= hInstance;
  wc.hIcon:= 0;
  wc.hCursor:= 0;
  wc.hbrBackground:= GetStockObject(BLACK_BRUSH);
  wc.lpszMenuName:= nil;
  wc.lpszClassName:= APP_NAME;

  if RegisterClass(wc) = 0 then Halt(1);

  // Create the main window.
  g_hwnd:= CreateWindow(APP_NAME, APP_NAME, WS_OVERLAPPEDWINDOW,
    Integer(CW_USEDEFAULT), Integer(CW_USEDEFAULT), 800, 600, GetDesktopWindow,
    0, hInstance, nil);

  // If the main window cannot be created, terminate
  // the application.
  if g_hwnd = 0 then Halt(1);

  pEnum:= Direct3DCreate8(D3D_SDK_VERSION);
  if (pEnum = nil) then Halt(1);

  hdc:= GetDC(g_hwnd);
  if hdc = 0 then Halt(1);

  d3dpp:= GetPresentParameters(hdc);
  hr:= pEnum.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, g_hwnd,
    D3DCREATE_SOFTWARE_VERTEXPROCESSING, d3dpp, dev);
  pEnum:= nil;

  ReleaseDC(g_hwnd, hdc);

  if (FAILED(hr)) then Halt(1);

  InitScene;
  InitFlash;

  ShowWindow(g_hwnd, SW_SHOWDEFAULT);
  UpdateWindow(g_hwnd);

  finished := False;

  CurrentTime := 0.0;
  QueryPerformanceFrequency(QFreq);
  // Main message loop:
  while not finished do
  begin
    if (PeekMessage(msg, 0, 0, 0, PM_REMOVE)) then // Check if there is a message for this window
    begin
      if (msg.message = WM_QUIT) then     // If WM_QUIT message received then we are done
        finished := True
      else
      begin                               // Else translate and dispatch the message to this window
    	  TranslateMessage(msg);
        DispatchMessage(msg);
      end;
    end
    else
    begin
      QueryPerformanceCounter(QTimerBefore);
      { draw scene }
      DrawScene;
      Sleep(5);
      { end draw scene }
      QueryPerformanceCounter(QTimerAfter);

      CurrentTime := CurrentTime + (QTimerAfter - QTimerBefore) / QFreq;
      DeltaTime := CurrentTime - LastTime;
      LastTime := CurrentTime;

      Inc(FPSCount);                      // Increment FPS Counter
      FrameTime := FrameTime + DeltaTime;

      if FrameTime > 1 then
      begin
        SetWindowText(g_hwnd, PChar(IntToStr(Round(FPSCount / FrameTime)) + ' FPS'));
        FrameTime := 0;
        FPSCount := 0;
      end;
    end;
  end;

  FreeFlash;
  DestroyWindow(g_hwnd);
  dev:= nil;
end.

